import gym
from gym import spaces
import numpy as np
import pandas as pd

class StockTradingEnv(gym.Env):
    metadata = {'render.modes': ['human']}

    def __init__(self, df):
        super(StockTradingEnv, self).__init__()

        self.df = df
        self.current_step = 0

        self.action_space = spaces.Discrete(3)  # 0: hold, 1: buy, 2: sell

        self.observation_space = spaces.Box(low=0, high=np.inf, shape=(6,), dtype=np.float32)

        self.balance = 100000  # Initial cash balance
        self.shares_held = 0
        self.total_profit = 0

    def reset(self):
        self.balance = 100000
        self.shares_held = 0
        self.total_profit = 0
        self.current_step = 0
        return self._next_observation()

    def _next_observation(self):
        frame = np.array([
            self.df.loc[self.current_step, 'Open'],
            self.df.loc[self.current_step, 'High'],
            self.df.loc[self.current_step, 'Low'],
            self.df.loc[self.current_step, 'Close'],
            self.df.loc[self.current_step, 'Volume'],
            self.balance,
        ])
        return frame

    def step(self, action):
        self.current_step += 1

        done = self.current_step >= len(self.df) - 1

        if action == 1 and self.balance > self.df.loc[self.current_step, 'Close']:  # Buy
            self.balance -= self.df.loc[self.current_step, 'Close']
            self.shares_held += 1
        elif action == 2 and self.shares_held > 0:  # Sell
            self.balance += self.df.loc[self.current_step, 'Close']
            self.shares_held -= 1
            self.total_profit += self.df.loc[self.current_step, 'Close']

        if done:
            reward = self.balance + self.shares_held * self.df.loc[self.current_step, 'Close'] - 100000
        else:
            reward = self.balance + self.shares_held * self.df.loc[self.current_step, 'Close'] - 100000

        return self._next_observation(), reward, done, {}

    def render(self, mode='human'):
        profit = self.balance + self.shares_held * self.df.loc[self.current_step, 'Close'] - 100000
        print(f'Step: {self.current_step}')
        print(f'Balance: {self.balance}')
        print(f'Shares held: {self.shares_held}')
        print(f'Profit: {profit}')
