import websocket
import json
import logging

# websocket_client.py

def connect_to_websocket():
    # 함수 구현 내용
    pass

# 만약 함수가 정의되어 있지 않다면 다음과 같이 추가합니다.


def on_message(ws, message):
    logging.info(f"Received message: {message}")
    try:
        message_data = json.loads(message)
        if 'header' in message_data and message_data['header']['tr_id'] == 'PINGPONG':
            logging.info("Received a ping-pong message. No action needed.")
            return

        if 'body' in message_data and 'output' in message_data['body']:
            stock_code = message_data['body']['output'].get('stock_code')
            if stock_code:
                logging.info(f"Processing stock code: {stock_code}")
                # Process stock data here
            else:
                logging.warning("No stock code found in the message")
        else:
            logging.warning("Unexpected message format")
    except json.JSONDecodeError:
        logging.error("Error decoding JSON message")

def on_error(ws, error):
    logging.error(f"Error: {error}")

def on_close(ws):
    logging.info("Connection closed")

def on_open(ws):
    logging.info("Websocket connected")
    ws.send(json.dumps({"header":{"tr_id":"PINGPONG"}}))

if __name__ == "__main__":
    websocket.enableTrace(True)
    ws = websocket.WebSocketApp("ws://ops.koreainvestment.com:21000",
                                on_open=on_open,
                                on_message=on_message,
                                on_error=on_error,
                                on_close=on_close)

    ws.run_forever()
